#include <stdio.h>
#include <time.h>
#include <math.h>

#include "chil_global.h"

#define MAX_TABLE_LENGTH 1000

static time_t calendar_time[MAX_TABLE_LENGTH];
static float calibration_value[MAX_TABLE_LENGTH];
static int nentries = 0;
/* Read a table of calibration data from a file. The data should look like this:
19990301     1.2    
19990401     1.2    
19990501     1.3    
19990601     1.4    
19990701     1.5    
19990801     1.7    
19990901     2.2    
19991001     3.0    
Returns the number of entries read (0 on failure) */
int 
CHILReadCalibrationTable(char *filename)
{
  int i = 0;
  FILE *file = fopen(filename, "r");
  struct tm date = {0, 0, 0, 0, 0, 0, 0, 0, 0};

  if (!file) {
    return 0;
  }
  
  while (i < MAX_TABLE_LENGTH) {
    int yyyymmdd;
    float cal;
    if (fscanf(file, "%d %f", &yyyymmdd, &cal) < 2) {
      break;
    }
    date.tm_year = yyyymmdd/10000;
    date.tm_mon = (yyyymmdd-date.tm_year*10000)/100;
    date.tm_mday = (yyyymmdd-date.tm_year*10000-date.tm_mon*100);

    date.tm_mon -= 1;  /* Non standard! */
    date.tm_year -= 1900; /* Non standard! */

    if ((calendar_time[i] = mktime(&date)) < 0) {
      fclose(file);
      return 0;
    }
    if (i > 0 && calendar_time[i] <= calendar_time[i-1]) {
      fclose(file);
      return 0;
    }

    calibration_value[i] = cal;
    i++;
  }
  fclose(file);
  return (nentries = i);
}

/* Interpolate a calibration value based on the date of
   interest. Returns 0 on success, 1 on failure. */
int
CHILGetCalibrationValue(int year, int month, int day, float *cal)
{
  time_t cal_time;
  struct tm date = {0, 0, 0, 0, 0, 0, 0, 0, 0};
  int i = 0;

  if (!nentries) {
    return 1;
  }

  date.tm_year = year - 1900;
  date.tm_mon = month - 1;
  date.tm_mday = day;

  cal_time = mktime(&date);

  if (cal_time < 0 && cal_time < calendar_time[0]
      && cal_time > calendar_time[nentries-1]) {
    return 1;
  }

  while (cal_time > calendar_time[++i]);
  
  *cal = ((calendar_time[i]-cal_time) * calibration_value[i-1]
	  + (cal_time-calendar_time[i-1]) * calibration_value[i])
    / (calendar_time[i] - calendar_time[i-1]);
  return 0;
}
