/* mextools.c
   A few usefull things to have when writing mex files
*/

/* This function converts the string arguments of a mex argument list 
   into a normal C argument list, 
   EXCEPT that argv[0] corresponds to the first argument, not the filename 
*/

#include <mex.h>

int
MexConvertStringArguments(int nrhs,const mxArray *prhs[], int *argc,
			  char ***argv,char **argt, char *mexname)
{
  int i,msize=0,nstrings=0;
  char *p_arg;

  *argc = nrhs + 1;

  for (i = 0; i < nrhs; i++) {
    if (mxGetClassID(prhs[i]) == mxCHAR_CLASS) {
      msize += mxGetM(prhs[i]) * mxGetN(prhs[i]) + 1;
    }
  }

  *argv = (void *) mxCalloc(msize + (*argc) * (sizeof(void *) + 1), 1);
  *argt = (void *) (*argv + *argc);
  p_arg = (void *) (*argt + *argc);

  *(*argv) = mexname;
  *(*argt) = mxCHAR_CLASS;

  for (i = 1; i < *argc; i++) {
    if ( ( *(*argt+i) = mxGetClassID(prhs[i-1])) == mxCHAR_CLASS) {
      msize = mxGetM(prhs[i-1]) * mxGetN(prhs[i-1]) + 1;
      mxGetString(prhs[i-1], *(*argv+i) = p_arg, msize);
      p_arg += msize;
      nstrings++;
    }
    else {
      *(*argv+i) = NULL;
    }
  }
  return nstrings;
}

void
MexFreeArguments(char **argv)
{
  mxFree(argv);
}

void
MexPrintArguments(int argc, char **argv)
{
  int i;
  for (i=0;i<argc;i++) {
    mexPrintf("Argument %d: ",i);
    if ( argv[i] == NULL)
      mexPrintf("not a string\n");
    else
      mexPrintf("`%s'\n",argv[i]);
  }
}


